@@ -1,5 +1,5 @@
 if (getRversion() >= "3.1.0") {
-  utils::globalVariables(c("checksumsFilename", "checksumsID", "id"))
+  utils::globalVariables(c("cacheId", "checksumsFilename", "checksumsID", "id"))
 }
 
 #' Basic tool for using cloud-based caching
@@ -29,15 +29,7 @@ cloudCheck <- function(toDigest, checksumsFileID = NULL, cloudFolderID = NULL) {
   suppressMessages(checksums <- cloudDownloadChecksums(checksumsFileID))
   hashExists <- checksums$cacheId == dig
   out <- if (isTRUE(any(hashExists))) {
-    objectFilename2 <- tempfile(fileext = ".rda");
-    a <- checksums[hashExists]$filesize
-    class(a) <- "object_size"
-    message("  downloading object from google drive; this could take a while; it is: ",
-            format(a, "auto"))
-    drive_download(as_id(checksums[hashExists, id]), path = objectFilename2, verbose = FALSE)
-    env <- new.env()
-    load(objectFilename2, envir = env)
-    as.list(env)[[1]]
+    cloudDownload(checksums[hashExists])
   } else {
     NULL
   }
@@ -195,20 +187,28 @@ cloudUpdateChecksums <- function(checksums, checksumsFileID) {
 cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
                        checksumsFileID = NULL, cloudFolderID = NULL) {
 
-  if (is.null(checksumsFileID))
-    if (is.null(cloudFolderID))
-      stop("You must supply a checksumsFileID or, if not supplied, a cloudFolderID where the",
-           "\n checksums file will be made")
   hasCopy <- FALSE
   hasCloudCopy <- FALSE
 
-  fnDetails <- .fnCleanup(FUN = eval(match.call(Cache, expand.dots = TRUE)$FUN),
-                          callingFun = "Cache", ...)
-  cacheRepo <- suppressMessages(.checkCacheRepo(fnDetails$modifiedDots, create = TRUE))
-  suppressMessages(archivist::createLocalRepo(cacheRepo))
+  if (is.null(list(...)$cacheRepo)) {
+    cacheRepos <- suppressMessages(.checkCacheRepo(list(...), create = TRUE))
+  } else {
+    cacheRepos <- lapply(list(...)$cacheRepo, function(repo) {
+      repo <- checkPath(repo, create = TRUE)
+    })
+  }
+  cacheRepo <- cacheRepos[[1]]
 
-  dots <- list(...)
+  #cacheRepo <- suppressMessages(.checkCacheRepo(list(...), create = TRUE))#fnDetails$modifiedDots, create = TRUE))
+  suppressMessages(archivist::createLocalRepo(cacheRepo))
   if (isTRUE(useCloud)) {
+
+    if (is.null(checksumsFileID))
+      if (is.null(cloudFolderID))
+        stop("You must supply a checksumsFileID or, if not supplied, a cloudFolderID where the",
+             "\n checksums file will be made")
+
+    dots <- list(...)
     if (!is.null(dots$omitArgs)) {
       dots <- .CacheDigestWithOmitArgs(sys.call(), envir = sys.frame(-1))
     }
@@ -233,6 +233,10 @@ cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
       hasCloudCopy <- any(cachedCopy$checksums$cacheId %in% cachedCopy$digest)
       # it may not have a cloud copy either, meaning it will have a NULL
       if (!is.null(cachedCopy$object)) {
+
+        fnDetails <- .fnCleanup(FUN = eval(match.call(Cache, expand.dots = TRUE)$FUN),
+                                callingFun = "Cache", ...)
+
         message("  writing to local Cache")
         objSize <- sum(unlist(objSize(cachedCopy)))
         preDigestUnlistTrunc <- unlist(.unlistToCharacter(dig$preDigest, 3))
@@ -273,7 +277,11 @@ cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
     }
   }
   out <- if (!hasCloudCopy || hasLocalCopy) {
-    Cache(..., cacheRepo = cacheRepo)
+    if ("cacheRepo" %in% names(list(...))) {
+      Cache(...)
+    } else {
+      Cache(..., cacheRepo = cacheRepo)
+    }
   } else {
     cachedCopy$object
   }
@@ -320,10 +328,25 @@ getChecksumsFileID <- function(cloudFolderID) {
 #'
 #' @inheritParams Cache
 #' @inheritParams cloudCache
+#' @inheritParams clearCache
 #' @param cacheRepo See \code{x} in \code{\link{showCache}}
+#' @param cacheIds If supplied, then only this/these cacheId objects
+#'   will be uploaded or deleted. Default is \code{NULL}, meaning do
+#'   full sync (i.e., match cloudFolder with local cacheRepo, constrained by
+#'   \code{delete} or \code{upload})
 #' @param delete Logical. If \code{TRUE}, the default, it will delete any objects
 #'   that are in \code{cloudFolderID} that are absent from local \code{cacheRepo}.
 #'   If \code{FALSE}, it will not delete objects.
+#' @param upload Logical. If \code{TRUE}, the default, it will upload any objects
+#'   identified by the internal \code{showCache(...)} call. See examples. If \code{FALSE},
+#'   then no files will be uploaded. Can be used in conjunction with \code{delete}
+#'   to create behaviours similar to \code{clearCache} and \code{keepCache}.
+#' @param download Logical. If \code{FALSE}, the default, then the function will
+#'   either delete the remote copy if \code{delete = TRUE} and there is no local
+#'   copy, or upload the local copy if \code{upload = TRUE} and there is a local
+#'   copy. If \code{TRUE}, then this will override \code{delete}, and download
+#'   to local machine if it exists remotely.
+#' @param ... Passed to \code{showCache} to get the artifacts to delete
 #'
 #' @export
 #' @importFrom crayon blue
@@ -334,6 +357,7 @@ getChecksumsFileID <- function(cloudFolderID) {
 #' @examples
 #' \dontrun{
 #'   #make a google drive folder
+#'   opts <- options("reproducible.cachePath" = tempdir(), "reproducible.ask" = FALSE)
 #'   library(googledrive)
 #'   newDir <- drive_mkdir("testFolder")
 #'   a <- Cache(rnorm, 1)
@@ -343,7 +367,7 @@ getChecksumsFileID <- function(cloudFolderID) {
 #'   cloudSyncCache(cloudFolderID = newDir$id)
 #'
 #'   # remove a local one
-#'   clearCache(userTags = CacheDigest(list(rnorm, 2))$outputHash, ask = FALSE)
+#'   clearCache(userTags = CacheDigest(list(rnorm, 2))$outputHash)
 #'
 #'   # Now will delete the object in the cloud that was just deleted locally
 #'   cloudSyncCache(cloudFolderID = newDir$id)
@@ -352,12 +376,69 @@ getChecksumsFileID <- function(cloudFolderID) {
 #'   clearCache(ask = FALSE)
 #'   cloudSyncCache(cloudFolderID = newDir$id)
 #'
+#'   #######################################################################
+#'   # use showCache args to have control ... on upload & delete NOTE difference!
+#'   #######################################################################
+#'   a <- Cache(rnorm, 1)
+#'   b <- Cache(rnorm, 2)
+#'   # only sync the one with rnorm, 2 as arguments
+#'   #   This CacheDigest is the same algorithm used by Cache
+#'   tag <- CacheDigest(list(rnorm, 2))$outputHash
+#'   cloudSyncCache(cloudFolderID = newDir$id, userTags = tag) # only syncs the one
+#'                                                             # that is identified
+#'                                                             # with userTags
+#'
+#'   cloudSyncCache(cloudFolderID = newDir$id) # sync any other ones
+#'
+#'   # Now clear an object locally -- next how to propagate this deletion to cloud
+#'   clearCache(userTags = tag)
+#'
+#'   # Add one more to local, so now local has 2 (a and d), cloud has 2 (a and b)
+#'   d <- Cache(rnorm, 4)
+#'
+#'   # DELETING IS DIFFERENT
+#'   # Doesn't quite work same way for deleting -- this tag is not in local Cache,
+#'   # so can't find it this way.
+#'   # This next line DOES THE WRONG THING -- IT DELETES EVERYTHING because there is
+#'   #         no entry in the local cache -- use cacheId arg instead -- see below
+#'   #    showCache(userTags = tags) shows empty
+#'   #    cloudSyncCache(cloudFolderID = newDir$id, userTags = tag)
+#'
+#'   # Only delete the one that was removed from local cache, set upload = FALSE,
+#'   #    leaving only 1 in cloud: a  -- this is still a sync, so, it will only
+#'   #    delete 1 file because local has 1 few files -- see next for just deleting 1 artifact
+#'   cloudSyncCache(cloudFolderID = newDir$id, upload = FALSE)
+#'   # Upload the d, because it is the only one in the localCache not in the cloudCache
+#'   cloudSyncCache(cloudFolderID = newDir$id)
+#'
+#'   f <- Cache(rnorm, 5)
+#'   g <- Cache(rnorm, 6)
+#'   # upload both
+#'   cloudSyncCache(cloudFolderID = newDir$id) # only syncs the one
+#'   tag5 <- CacheDigest(list(rnorm, 5))$outputHash # this is the same algorithm used by Cache
+#'   tag6 <- CacheDigest(list(rnorm, 6))$outputHash
+#'   clearCache(userTags = tag5)
+#'   clearCache(userTags = tag6)
+#'   # delete only one by tag
+#'   cloudSyncCache(cloudFolderID = newDir$id, cacheIds = tag5) # will delete only this obj in cloud
+#'   # delete another one by tag
+#'   cloudSyncCache(cloudFolderID = newDir$id, cacheIds = tag6)
+#'
+#'   # clean up
+#'   clearCache(ask = FALSE)
+#'   cloudSyncCache(cloudFolderID = newDir$id)
+#'
 #'   # To remove whole folder:
 #'   drive_rm(as_id(newDir$id))
+#'   options(opts)
 #' }
 cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
                            checksumsFileID = NULL, cloudFolderID = NULL,
-                           delete = TRUE, ...) {
+                           delete = TRUE, upload = TRUE, download = !delete,
+                           ask = getOption("reproducible.ask"),
+                           cacheIds = NULL,
+                           ...) {
+  if (isTRUE(download)) delete <- FALSE
   if (is.null(checksumsFileID)) {
     if (!is.null(cloudFolderID)) {
       checksumsFileID <- getChecksumsFileID(cloudFolderID)
@@ -370,14 +451,27 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
   suppressMessages(localCache <- showCache(x = cacheRepo, ...))
   out <- NULL
 
-  if (NROW(localCache)) {
-    uniqueCacheArtifacts <- unique(localCache$artifact)
+  needToDelete <- NULL
+  uniqueCacheArtifacts <- unique(localCache$artifact)
+  if (!is.null(uniqueCacheArtifacts))
     cacheIDs <- localCache[artifact %in% uniqueCacheArtifacts & tagKey == "cacheId"]$tagValue
+
+  if (!is.null(cacheIds)) {
+    cacheIdsExisting <- which(cacheIDs %in% cacheIds)
+    if (length(cacheIdsExisting) == 0) {
+      message("None of the supplied cacheIds exist locally: \n  ", paste(cacheIds, sep = "\n  "))
+    }
+    cacheIDs <- cacheIDs[cacheIdsExisting]
+  }
+
+  if (NROW(localCache) && isTRUE(upload)) {
     needToUpload <- !(cacheIDs %in% checksums$cacheId)
 
     if (sum(needToUpload) > 0) {
-      message(crayon::blue("Uploading", length(needToUpload), "files, ",
-                           "\n ", paste(uniqueCacheArtifacts[needToUpload], ".rda", sep = "",
+      message(crayon::blue("Uploading", sum(needToUpload), "files, ",
+                           "\n ", paste(cacheIDs[needToUpload], ".rda ( == ",
+                                        uniqueCacheArtifacts[needToUpload],
+                                        ".rda in local cache)", sep = "",
                                         collapse = "\n  ")))
       out <- lapply(uniqueCacheArtifacts[needToUpload], function(art) {
         cacheID <- localCache[artifact == art & tagKey == "cacheId"]$tagValue
@@ -388,42 +482,133 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
       })
       checksums <- rbindlist(
         list(checksums,
-             data.table(cacheId = cacheIDs, id = rbindlist(out)$id, time = as.character(Sys.time()),
-                        filesize = file.size(file.path(cacheRepo, "gallery", paste0(uniqueCacheArtifacts, ".rda"))))),
+             data.table(
+               cacheId = cacheIDs[needToUpload],
+               id = rbindlist(out)$id,
+               time = as.character(Sys.time()),
+               filesize = file.size(file.path(cacheRepo, "gallery",
+                                              paste0(uniqueCacheArtifacts[needToUpload], ".rda")))
+             )
+        ),
         use.names = TRUE, fill = TRUE)
       out <- rbindlist(out)
     }
 
-    needToDelete <- !(checksums$cacheId %in% cacheIDs)
   } else {
-    needToDelete <- rep(TRUE, NROW(checksums))
+    if (NROW(localCache) == 0) {
+      needToDelete <- rep(TRUE, NROW(checksums))
+    }
+  }
+  if (is.null(needToDelete)) {
+    if (!is.null(uniqueCacheArtifacts)) {
+      needToDelete <- if (!is.null(cacheIds)) {
+        out <- (checksums$cacheId %in% cacheIds)
+        if (all(!out)) {
+          message("That/those cacheId(s) are not in the cloudFolderID. Nothing to delete.")
+        }
+        out
+      } else {
+        !(checksums$cacheId %in% cacheIDs)
+      }
+
+    }
   }
 
   if (isTRUE(delete) && any(needToDelete)) {
-    rmRes <- drive_rm(as_id(checksums$id[needToDelete]), verbose = FALSE)
+    out <- if (isTRUE(ask)) {
+      readline(paste0("Are you sure you want to delete:\n  ",
+                      paste(checksums$cacheId[needToDelete], ".rda",
+                            sep = "", collapse = "\n  "), "\n Y or N: "))
+    } else {
+      "Y"
+    }
+    rmRes <- if (identical(tolower(out), "y")) {
+      drive_rm(as_id(checksums$id[needToDelete]), verbose = FALSE)
+    } else {
+      FALSE
+    }
     if (isTRUE(all(rmRes))) {
       message(crayon::blue(sum(needToDelete), "file(s) deleted:",
                            "\n ", paste(checksums$cacheId[needToDelete], ".rda",
                                         sep = "", collapse = "\n  ")))
+      checksums <- checksums[!needToDelete]
     }
-    checksums <- checksums[!needToDelete]
+  } else {
+    if (any(needToDelete)) {
+      needToDownload <- needToDelete
+      if (isTRUE(download)) {
+        if (!cacheIds %in% localCache[artifact %in% uniqueCacheArtifacts & tagKey == "cacheId"]$tagValue) {
+          out <- cloudDownload(checksums[needToDownload,])
+          objSize <- sum(unlist(objSize(out)))
+          #preDigestUnlistTrunc <- unlist(.unlistToCharacter(CacheDigest(list(out))$preDigest, 3))
+          userTags <- c(#if (!is.na(fnDetails$functionName))
+            #paste0("function:", fnDetails$functionName),
+            paste0("object.size:", objSize),
+            paste0("accessed:", Sys.time())#,
+            #paste("preDigest", names(preDigestUnlistTrunc),
+            #      preDigestUnlistTrunc, sep = ":")
+          )
+
+          written <- 0
+          suppressMessages(cacheRepo <- .checkCacheRepo(cacheRepo, create = TRUE))
+          setattr(out, "tags", paste0("cacheId:", checksums[needToDownload, cacheId]))
+          message("  Writing to local cacheRepo")
+          while (written >= 0) {
+            saved <- suppressWarnings(try(silent = TRUE,
+                                          archivist::saveToLocalRepo(
+                                            out,
+                                            repoDir = cacheRepo,
+                                            artifactName = NULL,
+                                            archiveData = FALSE,
+                                            archiveSessionInfo = FALSE,
+                                            archiveMiniature = FALSE,
+                                            rememberName = FALSE,
+                                            silent = TRUE,
+                                            userTags = userTags
+                                          )
+            ))
+
+            # This is for simultaneous write conflicts. SQLite on Windows can't handle them.
+            written <- if (is(saved, "try-error")) {
+              Sys.sleep(sum(runif(written + 1, 0.05, 0.1)))
+              written + 1
+            } else {
+              -1
+            }
+          }
+        } else {
+          message("  cacheIds already present locally")
+        }
+      } else {
+        message("There were files identified to delete to create full sync, but delete = FALSE")
+      }
+    }
+
   }
   suppressMessages(cloudUpdateChecksums(checksums, checksumsFileID))
   return(invisible(checksums))
 }
 
+
 cloudUploadFileAndChecksums <- function(objectFile, cloudFolderID, digest,
                                         checksums, checksumsFileID) {
-  uploadRes <- drive_upload(objectFile, path = as_id(cloudFolderID),
-                            name = paste0(digest, ".rda"), verbose = FALSE)
-  checksums <- rbindlist(
-    list(checksums,
-         data.table(cacheId = digest, id = uploadRes$id, time = as.character(Sys.time()),
-                    filesize = file.size(objectFile))), use.names = TRUE, fill = TRUE)
-  saveRDS(checksums, file = getOption("reproducible.cloudChecksumsFilename"))
-  res <- drive_update(as_id(checksumsFileID),
-               media = getOption("reproducible.cloudChecksumsFilename"),
-               verbose = FALSE)
+  uploadRes <- try(drive_upload(objectFile, path = as_id(cloudFolderID),
+                            name = paste0(digest, ".rda"), verbose = FALSE), silent = TRUE)
+  if (!is(uploadRes, "try-error")) {
+
+    checksums <- rbindlist(
+      list(checksums,
+           data.table(cacheId = digest, id = uploadRes$id, time = as.character(Sys.time()),
+                      filesize = file.size(objectFile))), use.names = TRUE, fill = TRUE)
+    saveRDS(checksums, file = getOption("reproducible.cloudChecksumsFilename"))
+    res <- drive_update(as_id(checksumsFileID),
+                        media = getOption("reproducible.cloudChecksumsFilename"),
+                        verbose = FALSE)
+  } else {
+    if (any(grepl("403", uploadRes))) {
+      message("No write access to cloudFolderID; not uploading cached copy")
+    }
+  }
   return(invisible(res))
 }
 
@@ -435,3 +620,19 @@ cloudUploadFileAndChecksums <- function(objectFile, cloudFolderID, digest,
   dots <- lapply(b, eval, envir = envir)
 
 }
+
+cloudDownload <- function(checksums) {
+  if (NROW(checksums) > 1) {
+    stop("Can only download 1 file at a time, currently")
+  }
+  objectFilename2 <- tempfile(fileext = ".rda");
+  a <- checksums$filesize
+  class(a) <- "object_size"
+  message("  downloading object from google drive; this could take a while; it is: ",
+          format(a, "auto"))
+  drive_download(as_id(checksums[, id]), path = objectFilename2, verbose = FALSE)
+  env <- new.env()
+  load(objectFilename2, envir = env)
+  as.list(env)[[1]]
+
+}
