@@ -18,6 +18,11 @@ postProcess.default <- function(x, ...) {
   x
 }
 
+#' @importFrom rlang eval_tidy
+postProcess.quosure <- function(x, ...) {
+  postProcess(rlang::eval_tidy(x), ...)
+}
+
 #' @export
 #' @rdname postProcess
 postProcess.list <- function(x, ...) {
@@ -144,89 +149,40 @@ postProcess.spatialObjects <- function(x, filename1 = NULL, filename2 = TRUE,
                                        useCache = getOption("reproducible.useCache", FALSE),
                                        ...) {
   # Test if user supplied wrong type of file for "studyArea", "rasterToMatch"
-  if (!is.null(studyArea) & !is(studyArea, "Spatial")) {
-    stop("The 'studyArea' provided is not a Spatial* object.")
+  x <- postProcessAllSpatial(x = x, studyArea = studyArea,
+                             rasterToMatch = rasterToMatch, useCache = useCache,
+                             filename1 =filename1, filename2 = filename2,
+                             useSAcrs = useSAcrs, overwrite = overwrite,
+                             ...)
+  return(x)
+}
+
+#' @export
+#' @example inst/examples/example_postProcess.R
+#' @importFrom sf st_as_sf
+#' @rdname postProcess
+postProcess.sf <- function(x, filename1 = NULL, filename2 = TRUE,
+                           studyArea = NULL, rasterToMatch = NULL,
+                           overwrite = getOption("reproducible.overwrite", TRUE),
+                           useSAcrs = FALSE,
+                           useCache = getOption("reproducible.useCache", FALSE),
+                           ...) {
+
+  # Test if user supplied wrong type of file for "studyArea", "rasterToMatch"
+  message("postProcess with sf class objects is still experimental")
+  if (!is.null(rasterToMatch)) {
+    stop("sf class objects are not yet working with rasterToMatch argument")
+  }
+  if (is(studyArea, "Spatial")) {
+    studyArea <- st_as_sf(studyArea)
   }
 
-  if (!is.null(rasterToMatch) & !is(rasterToMatch, "RasterLayer")) {
-    stop("The 'rasterToMatch' provided is not a Raster* object.")
-  }
+  x <- postProcessAllSpatial(x = x, studyArea = studyArea,
+                             rasterToMatch = rasterToMatch, useCache = useCache,
+                             filename1 =filename1, filename2 = filename2,
+                             useSAcrs = useSAcrs, overwrite = overwrite,
+                             ...)
 
-  dots <- list(...)
-
-  if (!is.null(dots$inputFilePath))  {
-    message("inputFilePath is being deprecated; use filename1")
-    filename1 <- dots$inputFilePath
-    dots$inputFilePath <- NULL
-  }
-
-  if (!is.null(dots$targetFilePath))  {
-    message("targetFilePath is being deprecated; use filename1.")
-    filename1 <- dots$targetFilePath
-    dots$targetFilePath <- NULL
-  }
-
-  if (!is.null(studyArea) || !is.null(rasterToMatch)) {
-    # fix errors if methods available
-    skipCacheMess <- "useCache is FALSE, skipping Cache"
-    skipCacheMess2 <- "No cacheRepo supplied"
-
-    ##################################
-    # cropInputs
-    ##################################
-    if (!is.null(rasterToMatch)) {
-      extRTM <- extent(rasterToMatch)
-      crsRTM <- crs(rasterToMatch)
-    } else {
-      extRTM <- NULL
-      crsRTM <- NULL
-    }
-
-    x <- Cache(cropInputs, x = x, studyArea = studyArea,
-               extentToMatch = extRTM,
-               extentCRS = crsRTM,
-               useCache = useCache, ...)
-
-    # cropInputs may have returned NULL if they don't overlap
-    if (!is.null(x)) {
-      objectName <- if (is.null(filename1)) NULL else basename(filename1)
-      x <- fixErrors(x = x, objectName = objectName,
-                     useCache = useCache, ...)
-
-      ##################################
-      # projectInputs
-      ##################################
-      targetCRS <- .getTargetCRS(useSAcrs, studyArea, rasterToMatch)
-
-      x <- Cache(projectInputs, x = x, targetCRS = targetCRS,
-                 rasterToMatch = rasterToMatch, useCache = useCache, ...)
-      # may need to fix again
-      x <- fixErrors(x = x, objectName = objectName,
-                     useCache = useCache, ...)
-
-      ##################################
-      # maskInputs
-      ##################################
-      x <- Cache(maskInputs, x = x, studyArea = studyArea,
-                 rasterToMatch = rasterToMatch, useCache = useCache, ...)
-
-      ##################################
-      # filename
-      ##################################
-      newFilename <- determineFilename(filename1 = filename1, filename2 = filename2, ...)
-
-      ##################################
-      # writeOutputs
-      ##################################
-      x <- do.call(writeOutputs, append(list(x = x, filename2 = newFilename,
-                                              overwrite = overwrite), dots))
-
-      if (dir.exists(file.path(raster::tmpDir(), "bigRasters"))) {
-        ## Delete gdalwarp results in temp
-        unlink(file.path(raster::tmpDir(), "bigRasters"), recursive = TRUE)
-      }
-    }
-  }
   return(x)
 }
 
@@ -274,8 +230,8 @@ cropInputs.default <- function(x, studyArea, rasterToMatch, ...) {
 #'                      \code{extentTomatch} instead of \code{rasterToMatch}
 #'
 #' @export
+#' @importFrom raster projectExtent tmpDir
 #' @rdname cropInputs
-#' @importFrom raster projectExtent
 cropInputs.spatialObjects <- function(x, studyArea = NULL, rasterToMatch = NULL,
                                       extentToMatch = NULL, extentCRS = NULL, ...) {
   if (!is.null(studyArea) || !is.null(rasterToMatch) || !is.null(extentToMatch)) {
@@ -297,7 +253,9 @@ cropInputs.spatialObjects <- function(x, studyArea = NULL, rasterToMatch = NULL,
         projectExtent(cropTo, crs(x))
       } else {
         if (is(studyArea, "Spatial")) {
-          spTransform(x = cropTo, CRSobj = crs(x))
+          theExtent <- as(extent(cropTo), "SpatialPolygons")
+          crs(theExtent) <- crs(cropTo)
+          raster::extent(spTransform(x = theExtent, CRSobj = crs(x)))
         } else {
           NULL
         }
@@ -311,11 +269,10 @@ cropInputs.spatialObjects <- function(x, studyArea = NULL, rasterToMatch = NULL,
         dots <- list(...)
         dots[.formalsNotInCurrentDots("crop", ...)] <- NULL
         if (canProcessInMemory(x, 4)) {
-        x <- do.call(raster::crop, args = append(list(x = x, y = cropExtent), dots))
+          x <- do.call(raster::crop, args = append(list(x = x, y = cropExtent), dots))
         } else {
-          x <- do.call(raster::crop, args = append(list(x = x,
-                                                        y = cropExtent,
-                                                        filename = paste0(tempfile(), ".tif")),
+          x <- do.call(raster::crop, args = append(list(x = x, y = cropExtent,
+                                                        filename = paste0(tempfile(tmpdir = tmpDir()), ".tif")),
                                                    dots))
         }
         if (is.null(x)) {
@@ -327,6 +284,62 @@ cropInputs.spatialObjects <- function(x, studyArea = NULL, rasterToMatch = NULL,
   return(x)
 }
 
+#' @export
+#' @importFrom raster crs extent projectExtent raster
+#' @importFrom sf st_crop st_crs st_transform
+#' @rdname cropInputs
+cropInputs.sf <- function(x, studyArea = NULL, rasterToMatch = NULL,
+                          extentToMatch = NULL, extentCRS = NULL, ...) {
+  message("cropInputs with sf class objects is still experimental")
+  if (!is.null(studyArea) || !is.null(rasterToMatch) || !is.null(extentToMatch)) {
+    if (!is.null(extentToMatch)) {
+      rasterToMatch <- raster(extentToMatch, crs = extentCRS)
+    }
+    cropTo <- if (!is.null(rasterToMatch)) {
+      rasterToMatch
+    } else {
+      studyArea
+    }
+
+    # have to project the extent to the x projection so crop will work -- this is temporary
+    #   once cropped, then cropExtent should be rm
+    cropExtent <- if (identical(crs(x), crs(cropTo))) {
+      extent(cropTo)
+    } else {
+      if (!is.null(rasterToMatch)) {
+        stop("Can't work with rasterToMatch and sf objects yet in cropInputs")
+        projectExtent(cropTo, crs(x))
+      } else {
+        if (is(studyArea, "sf")) {
+          sf::st_transform(x = cropTo, crs = st_crs(x))
+        } else if (is(studyArea, "Spatial")) {
+          sf::st_transform(x = sf::st_as_sf(cropTo), crs = st_crs(x))
+        } else {
+          NULL
+        }
+      }
+    }
+
+    if (!identical(crs(x), crs(cropExtent))) {
+      crs(cropExtent) <- crs(x)
+    }
+
+    if (!is.null(cropExtent)) {
+      # crop it
+      if (!identical(cropExtent, extent(x))) {
+        message("    cropping ...")
+        dots <- list(...)
+        dots[.formalsNotInCurrentDots("crop", ...)] <- NULL
+        x <- do.call(sf::st_crop, args = append(list(x = x, y = cropExtent), dots))
+        if (is.null(x)) {
+          message("    polygons do not intersect.")
+        }
+      }
+    }
+  }
+  return(x)
+}
+
 #' Do some minor error fixing
 #'
 #' These must be very common for this function to be useful. Currently, the only
@@ -353,6 +366,7 @@ fixErrors <- function(x, objectName, attemptErrorFixes = TRUE,
 
 #' @export
 #' @keywords internal
+#' @rdname fixErrors
 fixErrors.default <- function(x, objectName, attemptErrorFixes = TRUE,
                               useCache = getOption("reproducible.useCache", FALSE), ...) {
   x
@@ -367,6 +381,7 @@ fixErrors.default <- function(x, objectName, attemptErrorFixes = TRUE,
 #' @param x A \code{SpatialPolygons} object
 #' @inheritParams fixErrors
 #' @importFrom testthat capture_warnings
+#' @rdname fixErrors
 fixErrors.SpatialPolygons <- function(x, objectName = NULL,
                                       attemptErrorFixes = TRUE,
                                       useCache = getOption("reproducible.useCache", FALSE), ...) {
@@ -403,12 +418,51 @@ fixErrors.SpatialPolygons <- function(x, objectName = NULL,
   return(x)
 }
 
+#' @inheritParams fixErrors
+#' @param x A \code{SpatialPolygons} object
+#'
+#' @export
+#' @importFrom sf st_buffer st_geometry st_is_valid
+#' @importFrom testthat capture_warnings
+#' @rdname fixErrors
+fixErrors.sf <- function(x, objectName = NULL, attemptErrorFixes = TRUE,
+                         useCache = getOption("reproducible.useCache", FALSE), ...) {
+  if (attemptErrorFixes) {
+    if (is.null(objectName)) objectName = "SimpleFeature"
+    if (is(st_geometry(x), "sfc_MULTIPOLYGON") || is(st_geometry(x), "sfc_GEOMETRY")) {
+      message("Checking for errors in ", objectName)
+      if (suppressWarnings(any(!sf::st_is_valid(x)))) {
+        message("Found errors in ", objectName, ". Attempting to correct.")
+        warn <- capture_warnings(
+          x1 <- try(Cache(sf::st_buffer, x, dist = 0, useCache = useCache))
+        )
+
+        # prevent the warning about not projected, because we are buffering 0, which doesn't matter
+        warnAboutNotProjected <- startsWith(warn, "Spatial object is not projected; GEOS expects planar coordinates")
+        if (any(warnAboutNotProjected))
+          warn <- warn[!warnAboutNotProjected]
+        if (length(warn))
+          warning(warn)
+
+        if (is(x1, "try-error")) {
+          message("There are errors with ", objectName,
+                  ". Couldn't fix them with raster::buffer(..., width = 0)")
+        } else {
+          x <- x1
+          message("  Some or all of the errors fixed.")
+        }
+      } else {
+        message("  Found no errors.")
+      }
+    }
+  }
+  return(x)
+}
 
 #' Project \code{Raster*} or {Spatial*} or \code{sf} objects
 #'
 #' A simple wrapper around the various different tools for these GIS types.
 #'
-#' @export
 #' @param x A \code{Raster*}, \code{Spatial*} or \code{sf} object
 #' @param targetCRS The CRS of x at the end  of this function (i.e., the goal)
 #' @param ... Passed to \code{\link[raster]{projectRaster}}.
@@ -421,11 +475,13 @@ fixErrors.SpatialPolygons <- function(x, objectName = NULL,
 #'                      number of cores in the system, while an integer or rounded
 #'                      float will be passed as the exact number of cores to be used.
 #'
-#' @rdname projectInputs
+#' @return A file of the same type as starting, but with projection (and possibly
+#' other characteristics, including resolution, origin, extent if changed).
+#'
+#' @export
 #' @importFrom raster canProcessInMemory
-#' @return
-#' A file of the same type as starting, but with projection (and possibly other
-#' characteristics, including resolution, origin, extent if changed.
+#' @rdname projectInputs
+#'
 #' @example inst/examples/example_postProcess.R
 projectInputs <- function(x, targetCRS, ...) {
   UseMethod("projectInputs")
@@ -439,9 +495,9 @@ projectInputs.default <- function(x, targetCRS, ...) {
 
 #' @export
 #' @rdname projectInputs
-#' @importFrom parallel detectCores
 #' @importFrom fpCompare %==%
 #' @importFrom gdalUtils gdal_setInstallation gdalwarp
+#' @importFrom parallel detectCores
 #' @importFrom raster crs dataType res res<- dataType<-
 projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, cores = NULL, ...) {
   dots <- list(...)
@@ -452,7 +508,7 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
   }
 
   if (is.null(rasterToMatch) & is.null(targetCRS)) {
-    message("     no reprojecting because no rasterToMatch & useSAcrs are FALSE.")
+    message("     no reprojecting because no rasterToMatch & targetCRS are FALSE (or NULL).")
   } else if (is.null(rasterToMatch) & identical(crs(x), targetCRS)) {
     message("    no reprojecting because target CRS is same as input CRS.")
   } else {
@@ -468,14 +524,13 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
                !identical(extent(x), extent(rasterToMatch))) {
       doProjection <- TRUE
     }
-
-      if (doProjection) {
-        if (!canProcessInMemory(x, 4)) {
-          message("   large raster: reprojecting after writing to temp drive...")
-          #rasters need to go to same file so it can be unlinked at end without losing other temp files
-          tmpRasPath <- checkPath(file.path(raster::tmpDir(), "bigRasters"), create = TRUE)
-          tempSrcRaster <- file.path(tmpRasPath, "bigRasInput.tif")
-          tempDstRaster <- file.path(tmpRasPath, paste0(x@data@names,"a_reproj.tif")) #fails if x = stack
+    if (doProjection) {
+      if (!canProcessInMemory(x, 4)) {
+        message("   large raster: reprojecting after writing to temp drive...")
+        #rasters need to go to same file so it can be unlinked at end without losing other temp files
+        tmpRasPath <- checkPath(file.path(raster::tmpDir(), "bigRasters"), create = TRUE)
+        tempSrcRaster <- file.path(tmpRasPath, "bigRasInput.tif")
+        tempDstRaster <- file.path(tmpRasPath, paste0(x@data@names, "a_reproj.tif")) #fails if x = stack
 
         if (!is.null(rasterToMatch)) {
           tr <- res(rasterToMatch)
@@ -530,26 +585,26 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
           }
         }
 
-          dType <- assessDataType(raster(tempSrcRaster), type = "GDAL")
-          system(
-            paste0(paste0(getOption("gdalUtils_gdalPath")[[1]]$path, "gdalwarp", exe, " "),
-                   "-s_srs \"", as.character(raster::crs(raster::raster(tempSrcRaster))), "\"",
-                   " -t_srs \"", as.character(targetCRS), "\"",
-                   " -multi ", prll,
-                   "-ot ", dType,
-                   teRas,
-                   "-r ", dots$method,
-                   " -overwrite ",
-                   "-tr ", paste(tr, collapse = " "), " ",
-                   "\"", tempSrcRaster, "\"", " ",
-                   "\"", tempDstRaster, "\""),
-            wait = TRUE)
-          ##
-          x <- raster(tempDstRaster)
-          crs(x) <- targetCRS #sometimes the crs is correct but the character string is not identical
-          #file exists in temp drive. Can copy to filename2
-        } else {
-          origDataType <- dataType(x)
+        dType <- assessDataType(raster(tempSrcRaster), type = "GDAL")
+        system(
+          paste0(paste0(getOption("gdalUtils_gdalPath")[[1]]$path, "gdalwarp", exe, " "),
+                 "-s_srs \"", as.character(raster::crs(raster::raster(tempSrcRaster))), "\"",
+                 " -t_srs \"", as.character(targetCRS), "\"",
+                 " -multi ", prll,
+                 "-ot ", dType,
+                 teRas,
+                 "-r ", dots$method,
+                 " -overwrite ",
+                 "-tr ", paste(tr, collapse = " "), " ",
+                 "\"", tempSrcRaster, "\"", " ",
+                 "\"", tempDstRaster, "\""),
+          wait = TRUE)
+        ##
+        x <- raster(tempDstRaster)
+        crs(x) <- targetCRS #sometimes the crs is correct but the character string is not identical
+        #file exists in temp drive. Can copy to filename2
+      } else {
+        origDataType <- dataType(x)
 
         # Capture problems that projectRaster has with objects of class integers,
         #   which is different than if they are integers (i.e., a numeric class object)
@@ -562,8 +617,8 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
         if (isInteger) {
           if (!is.null(dots$method)) {
             if (dots$method != "ngb") {
-            warning("This raster layer has integer values; it will be reprojected to float. ",
-                    "Did you want to pass 'method = \"ngb\"'?")
+              warning("This raster layer has integer values; it will be reprojected to float. ",
+                      "Did you want to pass 'method = \"ngb\"'?")
             }
           }
         }
@@ -594,6 +649,10 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
             }
           }
         }
+        if (!identical(crs(x), targetCRS)) {
+          crs(x) <- targetCRS # sometimes the proj4string is rearranged, so they are not identical:
+                              #  they should be
+        }
 
         # return the integer class to the data in the raster object
         if (isTRUE(isInteger)) {
@@ -612,7 +671,6 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
     } else {
       message("    no reprojecting because target characteristics same as input Raster.")
     }
-
   }
 
   if (isFactorRaster) {
@@ -623,19 +681,29 @@ projectInputs.Raster <- function(x, targetCRS = NULL, rasterToMatch = NULL, core
 }
 
 #' @export
-#' @importFrom sf st_transform
+#' @importFrom sf st_buffer st_crs st_is st_is_valid st_transform
 #' @rdname projectInputs
 projectInputs.sf <- function(x, targetCRS, ...) {
-  warning("sf class objects not fully implemented. Use with projectInputs.sf caution.")
-  if (requireNamespace("sf")) {
-    if (any(sf::st_is(x, c("POLYGON", "MULTIPOLYGON"))) && !any(isValid <- sf::st_is_valid(x))) {
-      x[!isValid] <- sf::st_buffer(x[!isValid], dist = 0, ...)
-    }
+  if (!is.null(targetCRS)) {
+    warning("sf class objects not fully tested Use with caution.")
+    if (requireNamespace("sf")) {
+      if (any(sf::st_is(x, c("POLYGON", "MULTIPOLYGON"))) && !any(isValid <- sf::st_is_valid(x))) {
+        x[!isValid] <- sf::st_buffer(x[!isValid], dist = 0, ...)
+      }
 
-    x <- sf::st_transform(x = x, crs = sf::st_crs(targetCRS@projargs), ...)
-  } else {
-    stop("Please install sf package: https://github.com/r-spatial/sf")
+      if ("projargs" %in% slotNames(targetCRS) )
+        targetCRS <- sf::st_crs(targetCRS@projargs)
+      x <- sf::st_transform(x = x, crs = targetCRS, ...)
+      if (!identical(sf::st_crs(x), targetCRS)) {
+        sf::st_crs(x) <- targetCRS # sometimes the proj4string is rearranged, so they are not identical:
+        #  they should be
+      }
+
+    } else {
+      stop("Please install sf package: https://github.com/r-spatial/sf")
+    }
   }
+  x
 }
 
 #' @export
@@ -654,6 +722,10 @@ projectInputs.Spatial <- function(x, targetCRS, ...) {
       }
     }
     x <- spTransform(x = x, CRSobj = targetCRS)
+    if (!identical(crs(x), targetCRS)) {
+      crs(x) <- targetCRS # sometimes the proj4string is rearranged, so they are not identical:
+      #  they should be
+    }
   }
   x
 }
@@ -665,13 +737,15 @@ projectInputs.Spatial <- function(x, targetCRS, ...) {
 #' @inheritParams postProcess.spatialObjects
 #' @rdname postProcessHelpers
 #' @keywords internal
-.getTargetCRS <- function(useSAcrs, studyArea, rasterToMatch) {
-  targetCRS <- if (useSAcrs) {
-    crs(studyArea)
-  } else if (!is.null(rasterToMatch)) {
-    crs(rasterToMatch)
-  } else {
-    NULL # don't reproject a Raster if only has studyArea -- too lossy
+.getTargetCRS <- function(useSAcrs, studyArea, rasterToMatch, targetCRS = NULL) {
+  if (is.null(targetCRS)) {
+    targetCRS <- if (useSAcrs) {
+      crs(studyArea)
+    } else if (!is.null(rasterToMatch)) {
+      crs(rasterToMatch)
+    } else {
+      NULL # don't reproject a Raster if only has studyArea -- too lossy
+    }
   }
   targetCRS
 }
@@ -721,8 +795,10 @@ maskInputs.Raster <- function(x, studyArea, rasterToMatch, maskWithRTM = FALSE,
 maskInputs.Spatial <- function(x, studyArea, ...) {
   if (!is.null(studyArea)) {
     message("    intersecting ...")
-    studyArea <- raster::aggregate(studyArea, dissolve = TRUE)
-    studyArea <- spTransform(studyArea, CRSobj = crs(x))
+    if (NROW(studyArea) > 1)
+      studyArea <- raster::aggregate(studyArea, dissolve = TRUE)
+    if (!identical(crs(x), crs(studyArea)))
+      studyArea <- spTransform(studyArea, CRSobj = crs(x))
     suppressWarnings(studyArea <- fixErrors(studyArea, "studyArea"))
     # raster::intersect -- did weird things in case of SpatialPolygonsDataFrame
     #  specifically ecodistricts.shp . It created an invalid object with
@@ -741,6 +817,48 @@ maskInputs.Spatial <- function(x, studyArea, ...) {
       xTmp <- sf::st_join(st_as_sf(x), st_as_sf(studyArea), join = st_intersects)
       y <- as(xTmp, "Spatial")
     }
+    if (!identical(crs(y), crs(x))) {
+      crs(y) <- crs(x) # sometimes the proj4string is rearranged, so they are not identical:
+      #  they should be
+    }
+
+    return(y)
+  } else {
+    return(x)
+  }
+}
+
+#' @export
+#' @rdname maskInputs
+#' @importFrom sf st_as_sf st_combine st_geometry st_intersection st_intersects st_join st_sf
+maskInputs.sf <- function(x, studyArea, ...) {
+  if (!is.null(studyArea)) {
+    if (is(studyArea, "Spatial"))
+      studyArea <- sf::st_as_sf(studyArea)
+
+    message("maskInputs with sf class objects is still experimental")
+    message("    intersecting ...")
+    #studyArea <- raster::aggregate(studyArea, dissolve = TRUE)
+    if (!identical(st_crs(x), st_crs(studyArea)))
+      studyArea <- sf::st_transform(studyArea, crs = st_crs(x))
+    if (NROW(studyArea) > 1)
+      studyArea <- sf::st_combine(studyArea)
+
+    studyArea <- sf::st_sf(studyArea)
+    if (is(sf::st_geometry(x), "sfc_POINT")) {
+      y1 <- sf::st_intersects(x, studyArea)
+      y2 <- sapply(y1, function(x) length(x) == 1)
+      ## TODO: usevapply instead of sapply; sapply is not type-safe
+      #y2 <- vapply(y1, function(x) length(x) == 1, logical(1))
+      y <- x[y2,]
+    } else {
+      studyArea <- fixErrors(studyArea)
+      y <- sf::st_intersection(x, studyArea)
+    }
+    if (!identical(crs(y), crs(x))) {
+      crs(y) <- crs(x) # sometimes the proj4string is rearranged, so they are not identical:
+      #  they should be
+    }
 
     return(y)
   } else {
@@ -800,12 +918,12 @@ maskInputs.Spatial <- function(x, studyArea, ...) {
 #'  absolute or relative path and used as is if absolute or prepended with
 #'  \code{destinationPath} if provided, and if \code{filename2} is relative.
 #'
+#' @importFrom raster tmpDir
 #' @rdname determineFilename
 #' @example inst/examples/example_postProcess.R
 determineFilename <- function(filename2 = TRUE, filename1 = NULL,
                               destinationPath = getOption("reproducible.destinationPath"),
                               prefix = "Small", ...) {
-
   if (!is.null(filename2)) {
     dots <- list(...)
 
@@ -837,7 +955,7 @@ determineFilename <- function(filename2 = TRUE, filename1 = NULL,
     filename2 <- if (!identical(filename2, FALSE)) { # allow TRUE or path
       if (isTRUE(filename2) ) {
         if (is.null(filename1)) {
-          tmpfile <- basename(tempfile())
+          tmpfile <- basename(tempfile(tmpdir = tmpDir()))
           filename1 <- tmpfile
         }
         .prefix(filename1, prefix)
@@ -856,9 +974,9 @@ determineFilename <- function(filename2 = TRUE, filename1 = NULL,
       NULL
     }
     if (exists("tmpfile", inherits = FALSE)) {
-      message("Saving output to ", filename2, ". Specify filename1 or filename2 for more control")
+      message("Saving output to ", filename2, ". Specify filename1 or filename2 for more control",
+              "\n  or set filename2 to NULL to prevent saving to disk")
     }
-
   }
   filename2
 }
@@ -951,17 +1069,23 @@ writeOutputs.Spatial <- function(x, filename2 = NULL,
 }
 
 #' @importFrom sf st_write
+#' @importFrom tools file_ext
 #' @rdname writeOutputs
 writeOutputs.sf <- function(x, filename2 = NULL,
                             overwrite = getOption("reproducible.overwrite", FALSE),
                             ...) {
   if (!is.null(filename2)) {
-    if (requireNamespace("sf")) {
-      x <- sf::st_write(obj = x, delete_dsn = TRUE, dsn = filename2, delete_dsn = overwrite,
-                        ...)
-    } else {
-      stop("Please install sf package: https://github.com/r-spatial/sf")
+    if (!nzchar(tools::file_ext(filename2))) {
+      filename2 <- paste0(filename2, ".shp")
     }
+    if (identical(".", dirname(filename2))) {
+      filename2 <- normPath(filename2)
+    }
+    if (!all(file.exists(filename2)))
+      overwrite = FALSE
+
+    sf::st_write(obj = x, dsn = filename2, delete_dsn = overwrite,
+                 ...)
   }
   x
 }
@@ -1135,3 +1259,108 @@ assessDataTypeGDAL <- function(ras) {
 
   datatype
 }
+
+#' @importFrom rlang eval_tidy
+postProcessChecks <- function(studyArea, rasterToMatch, dots) {
+  if (!is.null(studyArea) & !is(studyArea, "Spatial")) {
+    if (!is.null(studyArea) & !is(studyArea, "sf")) {
+      stop("The 'studyArea' provided is not a Spatial* object.")
+    }
+  }
+
+  if (!is.null(rasterToMatch) & !is(rasterToMatch, "RasterLayer")) {
+    stop("The 'rasterToMatch' provided is not a Raster* object.")
+  }
+
+  filename1 <- NULL
+  if (!is.null(dots$inputFilePath))  {
+    message("inputFilePath is being deprecated; use filename1")
+    filename1 <- dots$inputFilePath
+    dots$inputFilePath <- NULL
+  }
+
+  if (!is.null(dots$targetFilePath))  {
+    message("targetFilePath is being deprecated; use filename1.")
+    filename1 <- dots$targetFilePath
+    dots$targetFilePath <- NULL
+  }
+  list(dots = dots, filename1 = filename1)
+}
+
+postProcessAllSpatial <- function(x, studyArea, rasterToMatch, useCache, filename1,
+                                  filename2, useSAcrs, overwrite, targetCRS = NULL, ...) {
+  dots <- list(...)
+
+  if (!is.null(studyArea)) if (is(studyArea, "quosure")) studyArea <- rlang::eval_tidy(studyArea)
+  if (!is.null(rasterToMatch)) if (is(rasterToMatch, "quosure")) rasterToMatch <- rlang::eval_tidy(rasterToMatch)
+
+  extraDots <- postProcessChecks(studyArea = studyArea, rasterToMatch = rasterToMatch, dots = dots)
+  dots <- extraDots$dots
+  if (!is.null(extraDots$filename1))
+    filename1 <- extraDots$filename1
+
+  if (!is.null(studyArea) || !is.null(rasterToMatch) ||
+      !is.null(targetCRS)) {
+    # fix errors if methods available
+    skipCacheMess <- "useCache is FALSE, skipping Cache"
+    skipCacheMess2 <- "No cacheRepo supplied"
+
+    ##################################
+    # cropInputs
+    ##################################
+    if (!is.null(rasterToMatch)) {
+      extRTM <- extent(rasterToMatch)
+      crsRTM <- crs(rasterToMatch)
+    } else {
+      extRTM <- NULL
+      crsRTM <- NULL
+    }
+
+    x <- Cache(cropInputs, x = x, studyArea = studyArea,
+               extentToMatch = extRTM,
+               extentCRS = crsRTM,
+               useCache = useCache, ...)
+
+    # cropInputs may have returned NULL if they don't overlap
+    if (!is.null(x)) {
+      objectName <- if (is.null(filename1)) NULL else basename(filename1)
+      x <- fixErrors(x = x, objectName = objectName,
+                     useCache = useCache, ...)
+
+      ##################################
+      # projectInputs
+      ##################################
+      targetCRS <- .getTargetCRS(useSAcrs, studyArea, rasterToMatch,
+                                 targetCRS)
+
+      x <- Cache(projectInputs, x = x, targetCRS = targetCRS,
+                 rasterToMatch = rasterToMatch, useCache = useCache, ...)
+      # may need to fix again
+      x <- fixErrors(x = x, objectName = objectName,
+                     useCache = useCache, ...)
+
+      ##################################
+      # maskInputs
+      ##################################
+      x <- Cache(maskInputs, x = x, studyArea = studyArea,
+                 rasterToMatch = rasterToMatch, useCache = useCache, ...)
+
+      ##################################
+      # filename
+      ##################################
+      newFilename <- determineFilename(filename1 = filename1, filename2 = filename2, ...)
+
+      ##################################
+      # writeOutputs
+      ##################################
+      x <- do.call(writeOutputs, append(list(x = quote(x), filename2 = newFilename,
+                                             overwrite = overwrite), dots))
+
+      if (dir.exists(file.path(raster::tmpDir(), "bigRasters"))) {
+        ## Delete gdalwarp results in temp
+        unlink(file.path(raster::tmpDir(), "bigRasters"), recursive = TRUE)
+      }
+    }
+  }
+  x
+}
