@@ -2,14 +2,12 @@
 #' Create reproducible digests of objects in R
 #'
 #' Not all aspects of R objects are captured by current hashing tools in R (e.g.
-#' \code{digest::digest}, \code{fastdigest::fastdigest}, \code{knitr} caching,
+#' \code{digest::digest}, \code{knitr} caching,
 #' \code{archivist::cache}). This is mostly because many objects have "transient"
-#' (e.g., functions have environments), or "disk-backed" features. This function
-#' allows for these accommodations to be made and uses \code{\link[fastdigest]{fastdigest}}
-#' internally.  Since
+#' (e.g., functions have environments), or "disk-backed" features. Since
 #' the goal of using reproducibility is to have tools that are not session specific,
 #' this function
-#' attempts to strip all session specific information so that the fastdigest
+#' attempts to strip all session specific information so that the digest
 #' works between sessions and operating systems. It is tested under many
 #' conditions and object types, there are bound to be others that don't
 #' work correctly.
@@ -31,7 +29,7 @@
 #' converted to a text representation via a call to \code{format(FUN)}.
 #'
 #' Objects contained within a list or environment are recursively hashed
-#' using \code{\link[fastdigest]{fastdigest}}, while removing all references to
+#' using \code{\link[digest]{digest}}, while removing all references to
 #' environments.
 #'
 #' Character strings are first assessed with \code{dir.exists} and \code{file.exists}
@@ -52,7 +50,6 @@
 #' @return A hash i.e., digest of the object passed in.
 #'
 #' @seealso \code{\link[archivist]{cache}}.
-#' @seealso \code{\link[fastdigest]{fastdigest}}.
 #'
 #' @author Eliot McIntire
 #' @export
@@ -69,8 +66,8 @@
 #' save(a, file = tmpfile2)
 #'
 #' # treats as character string, so 2 filenames are different
-#' fastdigest::fastdigest(tmpfile1)
-#' fastdigest::fastdigest(tmpfile2)
+#' digest::digest(tmpfile1)
+#' digest::digest(tmpfile2)
 #'
 #' # tests to see whether character string is representing a file
 #' .robustDigest(tmpfile1)
@@ -95,8 +92,8 @@
 #'
 #' digest::digest(r1)
 #' digest::digest(r2) # different
-#' fastdigest::fastdigest(r1)
-#' fastdigest::fastdigest(r2) # different
+#' digest::digest(r1)
+#' digest::digest(r2) # different
 #' .robustDigest(r1)
 #' .robustDigest(r2) # same... data are the same in the file
 #'
@@ -104,11 +101,11 @@
 #' .robustDigest(r)
 #' .robustDigest(r1) # different
 #'
-setGeneric(".robustDigest", function(object, objects,
-                                    length = getOption("reproducible.length", Inf),
-                                    algo = "xxhash64",
-                                    quick = getOption("reproducible.quick", FALSE),
-                                    classOptions = list()) {
+setGeneric(".robustDigest", function(object, .objects,
+                                     length = getOption("reproducible.length", Inf),
+                                     algo = "xxhash64",
+                                     quick = getOption("reproducible.quick", FALSE),
+                                     classOptions = list(), ...) {
   standardGeneric(".robustDigest")
 })
 
@@ -117,10 +114,13 @@ setGeneric(".robustDigest", function(object, objects,
 setMethod(
   ".robustDigest",
   signature = "ANY",
-  definition = function(object, objects, length, algo, quick,
+  definition = function(object, .objects, length, algo, quick,
                         classOptions) {
     object <- .removeCacheAtts(object)
-    fastdigest(object)
+    if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+      digest(object, algo = algo)
+    else
+      fastdigest(object)
 })
 
 #' @import parallel
@@ -131,10 +131,12 @@ setOldClass("cluster")
 setMethod(
   ".robustDigest",
   signature = "cluster",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
-    object <- .removeCacheAtts(object)
-    fastdigest(NULL)
+  definition = function(object, .objects, length, algo, quick, classOptions) {
+    #object <- .removeCacheAtts(object)
+    if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+      digest(NULL, algo = algo)
+    else
+      fastdigest(NULL)
 })
 
 #' @rdname robustDigest
@@ -142,10 +144,8 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "function",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
-    object <- .removeCacheAtts(object)
-    fastdigest(format(object))
+  definition = function(object, .objects, length, algo, quick, classOptions) {
+    .robustDigestFormatOnly(object, algo = algo)
 })
 
 #' @rdname robustDigest
@@ -153,10 +153,8 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "expression",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
-    object <- .removeCacheAtts(object)
-    fastdigest(format(object))
+  definition = function(object, .objects, length, algo, quick, classOptions) {
+    .robustDigestFormatOnly(object, algo = algo)
 })
 
 #' @rdname robustDigest
@@ -164,38 +162,38 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "character",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
+  definition = function(object, .objects, length, algo, quick, classOptions) {
     object <- .removeCacheAtts(object)
 
-  if (!quick) {
-      # if (any(unlist(lapply(object, dir.exists)))) {
-      #   bbb <<- bbb + 1
-      #   unlist(lapply(object, function(x) {
-      #     if (dir.exists(x)) {
-      #       fastdigest(basename(x))
-      #     } else {
-      #       fastdigest(x)
-      #     }
-      #   }))
-      #} else
-    #aaa <<- aaa + length(object)
-      if (any(unlist(lapply(object, file.exists)))) {
-        unlist(lapply(object, function(x) {
-          if (dir.exists(x)) {
-            fastdigest(basename(x))
-          } else if (file.exists(x)) {
-            digest::digest(file = x, length = length, algo = algo)
-          } else {
-            fastdigest(x)
-          }
-        }))
+    if (!quick) {
+        if (any(unlist(lapply(object, file.exists)))) {
+          unlist(lapply(object, function(x) {
+            if (dir.exists(x)) {
+              if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+                digest(basename(x), algo = algo)
+              else
+                fastdigest(basename(x))
+            } else if (file.exists(x)) {
+                digest(file = x, length = length, algo = algo)
+            } else {
+              if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+                digest(x, algo = algo)
+              else
+                fastdigest(x)
+            }
+          }))
+        } else {
+          if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+            digest(object, algo = algo)
+          else
+            fastdigest(object)
+        }
       } else {
-        fastdigest(object)
+        if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+          digest(object, algo = algo)
+        else
+          fastdigest(object)
       }
-    } else {
-      fastdigest(object)
-    }
 })
 
 #' @rdname robustDigest
@@ -203,8 +201,7 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "Path",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
+  definition = function(object, .objects, length, algo, quick, classOptions) {
     nParentDirs <- attr(object, "nParentDirs")
     object <- .removeCacheAtts(object)
 
@@ -220,11 +217,17 @@ setMethod(
           digest::digest(file = x, length = length, algo = algo)
         } else {
           # just do file basename as a character string, if file does not exist
-          fastdigest(.basenames(x, nParentDirs))
+          if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+            digest(.basenames(x, nParentDirs), algo = algo)
+          else
+            fastdigest(.basenames(x, nParentDirs))
         }
       })
     } else {
-      fastdigest(.basenames(object, nParentDirs))
+      if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+        digest(.basenames(object, nParentDirs), algo = algo)
+      else
+        fastdigest(.basenames(object, nParentDirs))
     }
 })
 
@@ -233,10 +236,9 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "environment",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
+  definition = function(object, .objects, length, algo, quick, classOptions) {
     object <- .removeCacheAtts(object)
-    .robustDigest(as.list(object, all.names = TRUE), objects = objects,
+    .robustDigest(as.list(object, all.names = TRUE), .objects = .objects,
                  length = length,
                  algo = algo, quick = quick)
 })
@@ -246,25 +248,35 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "list",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
+  definition = function(object, .objects, length, algo, quick, classOptions) {
     object <- .removeCacheAtts(object)
     lapply(.sortDotsUnderscoreFirst(object), function(x) {
-      .robustDigest(object = x, objects = objects,
+      .robustDigest(object = x, .objects = .objects,
                    length = length,
                    algo = algo, quick = quick)
     })
 })
 
-
+#' @rdname robustDigest
+#' @export
+setMethod(
+  ".robustDigest",
+  signature = "data.frame",
+  definition = function(object, .objects, length, algo, quick, classOptions) {
+    #  Need a specific method for data.frame or else it get "list" method, which is wrong
+    object <- .removeCacheAtts(object)
+    if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+      digest(object, algo = algo)
+    else
+      fastdigest(object)
+})
 
 #' @rdname robustDigest
 #' @export
 setMethod(
   ".robustDigest",
   signature = "Raster",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
+  definition = function(object, .objects, length, algo, quick, classOptions) {
     object <- .removeCacheAtts(object)
 
     if (is(object, "RasterStack")) {
@@ -287,8 +299,7 @@ setMethod(
 setMethod(
   ".robustDigest",
   signature = "Spatial",
-  definition = function(object, objects, length, algo, quick,
-                        classOptions) {
+  definition = function(object, .objects, length, algo, quick, classOptions) {
     object <- .removeCacheAtts(object)
 
   if (is(object, "SpatialPoints")) {
@@ -307,7 +318,11 @@ setMethod(
       }
     }
 
-    return(fastdigest(aaa))
+    #
+    if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+      digest(aaa, algo = algo)
+    else
+      fastdigest(aaa)
 })
 
 .basenames <- function(object, nParentDirs) {
@@ -335,3 +350,12 @@ setMethod(
   setattr(x, "call", NULL)
   x
 }
+
+.robustDigestFormatOnly <- function(object, .objects, length, algo, quick,
+                               classOptions) {
+  object <- .removeCacheAtts(object)
+  if (isTRUE(getOption("reproducible.useNewDigestAlgorithm")))
+    digest(format(object), algo = algo)
+  else
+    fastdigest(format(object))
+}
